const fs = require('fs');
const path = require('path');

const requestUtils = require('../../../utils/requestUtils');
const sleepUtils = require('../../../utils/sleepUtils');
const dataManager = require('../dataManager');

const dbUtils = global.dbUtils;

// 正常应该查不出记录才对
/*
SELECT * FROM album WHERE (full_description = '' or full_description is null) and description like '%专辑《%》，简介：%' and description not regexp '^.*?专辑《.*?》，简介：([:space:]*?|[ ﻿]*?)。，更多.*$'
 */

async function fetchAll({ args = {}, isUpdate = false }) {
    console.log("start fetching albums ...");
    let albumIds = await dataManager.album.getIdsToFetch(args, isUpdate);
    console.log(`albumIds was fetched, count: ${albumIds.length}`);
    for (let i = 0; i < albumIds.length; i++) {
        await global.checkIsExit();
        const albumId = albumIds[i];
        console.log(`${i + 1}/${albumIds.length} | album: ${albumId} | ${args.min || "?"}-${args.max || "?"}`);
        try {
            await fetch({ albumId: albumId, update: isUpdate });
        } catch (err) {
            console.error(err);
        }
        await sleepUtils.sleep(global.sleepTime);
    }
}

// 获取专辑详情
async function fetch({ albumId, debug = false, update = false }) {
    let result = await dbUtils.query('SELECT count(*) as count FROM album WHERE album_id = ?', [albumId]);
    if (!debug && !update && result[0].count > 0) {
        console.log(`数据库中已有数据，跳过 albumId: ${albumId}`);
        // 从待爬取表中删除记录
        await dataManager.wait_fetch.deleteCollection("album", [albumId]);
        return;
    } else if (update && result[0].count == 0) {
        console.log(`数据库中沒有数据，跳过 albumId: ${albumId}`);
        return;
    }

    let url = `https://music.163.com/album?id=${albumId}`;
    try {
        // var html = fs.readFileSync(path.join(__dirname, "../../temp", `album-${albumId}.html`), 'utf8');
        var html = await requestUtils.getApiResult(url, { timeout: 3000 });
        // fs.writeFileSync(path.join(__dirname, "../../temp", `album-${albumId}.html`), html);
    } catch (errors) {
        console.error(errors);
        return;
    }

    if (html.includes(`<p class="note s-fc3">很抱歉，你要查找的网页找不到</p>`)) {
        // let deleteResult = await dbUtils.query('DELETE FROM song_album_relation WHERE album_id = ?', [albumId]);
        // console.log(`album: ${albumId} 不存在，从song_album_relation表中删除. affectedRows: ${deleteResult.affectedRows}`);
        console.log(`album: ${albumId} 不存在，插入空数据`);
        dbUtils.query('INSERT IGNORE INTO album SET ?', {
            album_id: albumId,
            title: '',
            description: '',
            image: '',
            pub_date: '',
            version: 1
        });
        return;
    }

    // 正则匹配
    let albumInfoDict;
    let regExResult = /\<script type\=\"application\/ld\+json\"\>([\S\s]*?)\<\/script\>/.exec(html);
    if (regExResult) {
        let albumInfoJSONString = regExResult[1];
        albumInfoDict = JSON.parse(albumInfoJSONString);
    } else {
        // 极少数album没有 <script type="application/ld+json">......</script> 这块内容
        // 例如：71715963，71738618。这些album都没有标题，所以下面这三项就直接保留空字符串
        albumInfoDict = {
            "title": "",
            "description": "",
            "pubDate": ""
        }
    }
    // console.log(albumInfoDict);

    // 发行公司
    let company = null;
    if (html.includes(`<p class="intr"><b>发行公司：`)) {
        try {
            // 注意 <b>发行公司：</b> 后面有可能只有一个换行 而没有内容
            company = /<p class="intr"><b>发行公司：<\/b>\n(.*?)\n?<\/p>/.exec(html)[1];
            company = company.trim();
        } catch (e) {
            // 解析出错
            await dbUtils.query('INSERT INTO log (`id`, `name`, `msg`) VALUES (?, ?, ?)', [albumId, 'album_fetch', `company 正则失败\n${e.message}`]);
            return;
        }
    }

    // 专辑详细简介
    let fullDescription = null;
    if (html.includes(`<div id="album-desc-more" class="f-hide">`)) {
        // 比较长 有点击展开按钮
        try {
            fullDescription = /<div id="album-desc-more" class="f-hide">([\S\s]*?)<\/div>/.exec(html)[1];
            fullDescription = fullDescription.replace(/<p class="f-brk">\n/g, '').replace(/<\/p>\n/g, '').trim();
        } catch (e) {
            // 解析出错
            await dbUtils.query('INSERT INTO log (`id`, `name`, `msg`) VALUES (?, ?, ?)', [albumId, 'album_fetch', `fullDescription 1 正则失败\n${e.message}`]);
            return;
        }
    } else if (html.includes(`<div id="album-desc-dot" class="f-brk">`)) {
        // 比较短 无点击展开按钮
        try {
            fullDescription = /<div id="album-desc-dot" class="f-brk">([\S\s]*?)<\/div>/.exec(html)[1];
            fullDescription = fullDescription.replace(/<p>/g, '').replace(/<\/p>/g, '').trim();
        } catch (e) {
            // 解析出错
            await dbUtils.query('INSERT INTO log (`id`, `name`, `msg`) VALUES (?, ?, ?)', [albumId, 'album_fetch', `fullDescription 2 正则失败\n${e.message}`]);
            return;
        }
    }

    // 极个别album下面的image正则找不到，例如：73973625 74009959
    let image = "";
    try {
        image = /<meta property="og:image" content="http:\/\/p.\.music\.126\.net\/(.*?)" \/>/.exec(html)[1];
    } catch (e) { }
    let songListJSONString = "";
    try {
        songListJSONString = /<textarea id="song-list-pre-data" style="display:none;">(.*?)<\/textarea>/.exec(html)[1];
    } catch (e) { }
    let songList = JSON.parse(songListJSONString);
    let songIds = songList.map(song => Number(song.id));

    let albumInfo = {
        album_id: albumId,
        title: albumInfoDict.title,
        image: image,
        description: albumInfoDict.description,
        full_description: fullDescription,
        pub_date: albumInfoDict.pubDate,
        company: company,
        version: 1
    };
    // console.log("albumInfo", albumInfo);

    // 插入待爬取表
    await dataManager.wait_check.insert("song", songIds);
    await dataManager.wait_check.insert("lyric", songIds);
    await dataManager.wait_check.insert("comment", songIds);

    // 插入关联关系
    if (albumId > 0) {
        let songAlbumRel = songIds.map(songId => [songId, albumId]);
        await dataManager.song_album.insertCollection(songAlbumRel);
    }

    // 插入数据
    if (update) {
        await dataManager.album.update(albumId, albumInfo);
    } else {
        await dataManager.album.insert(albumInfo);
    }

    // 从待爬取表中删除记录
    await dataManager.wait_fetch.deleteCollection("album", [albumId]);
}

module.exports = {
    fetch: fetch,
    fetchAll: fetchAll,
}