const fs = require('fs');
const path = require('path');

const requestUtils = require('../../../utils/requestUtils');
const sleepUtils = require('../../../utils/sleepUtils');
const dataManager = require('../dataManager');

const dbUtils = global.dbUtils;

// 从数据库中查出还缺少的歌手，并进行爬取
async function fetchAll({ args = {} }) {
    console.log("start fetching artists ...");
    let artistIds = await dataManager.artist.getIdsToFetch(args);
    console.log(`artistIds was fetched, count: ${artistIds.length}`);
    for (let i = 0; i < artistIds.length; i++) {
        await global.checkIsExit();
        const artistId = artistIds[i];
        console.log(`${i + 1}/${artistIds.length} | artist: ${artistId} | ${args.min || "?"}-${args.max || "?"}`);
        try {
            await fetch({ artistId: artistId });
        } catch (err) {
            console.error(err);
        }
        await sleepUtils.sleep(global.sleepTime);
    }
}

// 获取音乐人详情
async function fetch({ artistId, debug = false }) {
    let result = await dbUtils.query('SELECT count(*) as count FROM artist WHERE artist_id = ?', [artistId]);
    if (result[0].count > 0 && !debug) {
        console.log(`数据库中已有数据，跳过 artistId: ${artistId}`);
        // 从待爬取表中删除记录
        await dataManager.wait_fetch.deleteCollection("artist", [artistId]);
        return;
    }

    let url = `https://music.163.com/artist?id=${artistId}`;
    try {
        // var html = fs.readFileSync(path.join(__dirname, "../../temp", `artist-${artistId}.html`), 'utf8');
        var html = await requestUtils.getApiResult(url, { timeout: 3000 });
        // fs.writeFileSync(path.join(__dirname, "../../temp", `artist-${artistId}.html`), html);
    } catch (errors) {
        console.error(errors);
        return;
    }

    if (html.includes(`<p class="note s-fc3">很抱歉，你要查找的网页找不到</p>`)) {
        // let deleteResult = await dbUtils.query('DELETE FROM song_artist_relation WHERE artist_id = ?', [artistId]);
        // console.log(`artist: ${artistId} 不存在，从song_artist_relation表中删除. affectedRows: ${deleteResult.affectedRows}`);
        console.log(`artist: ${artistId} 不存在，插入空数据`);
        dbUtils.query('INSERT IGNORE INTO artist SET ?', {
            artist_id: artistId,
            title: '',
            description: '',
            image: '',
            pub_date: '',
        });
        return;
    }

    // 正则匹配
    let artistInfoDict;
    let regExResult = /\<script type\=\"application\/ld\+json\"\>([\S\s]*?)\<\/script\>/.exec(html);
    if (regExResult) {
        let artistInfoJSONString = regExResult[1];
        artistInfoDict = JSON.parse(artistInfoJSONString);
    } else {
        // 极少数artist没有 <script type="application/ld+json">......</script> 这块内容
        // 例如：29488473，29717445。这些artist都没有标题，所以下面这三项就直接保留空字符串
        artistInfoDict = {
            "title": "",
            "description": "",
            "pubDate": ""
        }
    }
    // console.log(artistInfoDict);

    let image = /<meta property="og:image" content="http:\/\/p.\.music\.126\.net\/(.*?)" \/>/.exec(html)[1];
    let songIds = [];
    try {
        let songListJSONString = /<textarea id="song-list-pre-data" style="display:none;">(.*?)<\/textarea>/.exec(html)[1];
        let songList = JSON.parse(songListJSONString);
        songIds = songList.map(song => Number(song.id));
    } catch (error) {
        // 可能是歌手下面没有音乐 例如：https://music.163.com/#/artist?id=30032762
    }

    let artistInfo = {
        artist_id: artistId,
        title: artistInfoDict.title,
        image: image,
        description: artistInfoDict.description,
        pub_date: artistInfoDict.pubDate
    };
    // console.log("artistInfo", artistInfo);

    // 插入待爬取表
    await dataManager.wait_check.insert("song", songIds);
    await dataManager.wait_check.insert("lyric", songIds);
    await dataManager.wait_check.insert("comment", songIds);

    // 插入关联关系
    if (artistId > 0) {
        let songArtistRel = songIds.map(songId => [songId, artistId]);
        await dataManager.song_artist.insertCollection(songArtistRel);
    }

    // 插入数据
    await dataManager.artist.insert(artistInfo);

    // 从待爬取表中删除记录
    await dataManager.wait_fetch.deleteCollection("artist", [artistId]);
}

module.exports = {
    fetch: fetch,
    fetchAll: fetchAll,
}