const fs = require('fs');
const path = require('path');

const sleepUtils = require('../../../utils/sleepUtils');
const dataManager = require('../dataManager');

const dbUtils = global.dbUtils;

// refer:
// https://neteasecloudmusicapi-docs.4everland.app/
// https://github.com/Binaryify/NeteaseCloudMusicApi
const { comment_music } = require('NeteaseCloudMusicApi');

async function fetchAll({ args = {} }) {
    console.log("start fetching comment ...");
    // // 首先将需要爬取的song_id导入comment_progress表
    // await dbUtils.query(`
    //     INSERT IGNORE INTO comment_progress ( song_id )
    //     SELECT song_id FROM wait_fetch_comment WHERE song_id NOT IN ( SELECT song_id FROM comment_progress )
    // `, []);
    let songIds = await dataManager.comment.getIdsToFetch(args);
    console.log(`songIds was fetched, count: ${songIds.length}`);
    if (songIds.length == 0) {
        console.log("完成！");
        process.exit(0);
    }
    for (let i = 0; i < songIds.length; i++) {
        await global.checkIsExit();
        const songId = songIds[i];
        console.log(`${i + 1}/${songIds.length} | comment: ${songId} | ${args.min || "?"}-${args.max || "?"}`);
        try {
            await fetch({ songId: songId });
        } catch (err) {
            console.error(err);
        }
        await sleepUtils.sleep(global.sleepTime);
    }
}

// 获取评论详情
async function fetch({ songId }) {
    // 首先查询有无正在爬取中的记录
    var commentProgress = await dbUtils.query(`
        SELECT * FROM comment_progress WHERE song_id = ? and current_status != 2 LIMIT 1
    `, [songId]);
    if (commentProgress.length == 0) {
        console.log('No commentProgress found, song_id:', songId);
        return;
    }
    var item = commentProgress[0];
    var progress = {
        maxTime: item.max_time,
        minTime: item.min_time,
        currentTime: item.current_time,
        currentStatus: item.current_status,
        total: item.total,
    };
    // https://neteasecloudmusicapi-docs.4everland.app/#/?id=%e6%ad%8c%e6%9b%b2%e8%af%84%e8%ae%ba
    var queryParams = {
        id: songId,
        limit: 20,
        // before: undefined,
    };
    if (progress.currentTime != 0)
        queryParams.before = progress.currentTime;

    let isFinish = false; let pageCount = 0;
    while (!isFinish) {
        await global.checkIsExit();
        console.log(`comment: ${songId}, page: ${++pageCount}, currentTime: ${progress.currentTime} | ${new Date(progress.currentTime + 8 * 3600 * 1000).toISOString().replace("T", " ").replace("Z", "")}`);

        // 是否是第一页
        let isFirstPage = progress.currentStatus === 0;

        try {
            // console.log(progress, queryParams);
            // console.log("await comment_music")
            var commentResult = await comment_music(queryParams);
            // console.log("finish await comment_music")
            // fs.writeFileSync(path.join(__dirname, "../../temp", `comment-${songId}-${pageCount}.json`), JSON.stringify(commentResult));
        } catch (errors) {
            console.error("error", errors);
            await sleepUtils.sleep(10 * 1000);
            continue;
        }

        var topComments = commentResult.body.topComments || [];
        var hotComments = commentResult.body.hotComments || [];
        var comments = commentResult.body.comments || [];

        var commentInfoList = [
            ...topComments.map(comment => getCommitInfoForInsert(songId, comment, 2)),
            ...hotComments.map(comment => getCommitInfoForInsert(songId, comment, 1)),
            ...comments.map(comment => getCommitInfoForInsert(songId, comment, 0))
        ];
        var userInfoList = [...topComments, ...hotComments, ...comments]
            .map(comment => comment.user).filter(user => !!user).map(getUserInfoForInsert);

        // console.log(commentInfoList);
        // console.log(userInfoList);

        console.log("dataManager.comment.insertCollection & dataManager.user.insertCollection")
        let p1 = dataManager.comment.insertCollection(commentInfoList.map(commentInfo => [
            commentInfo.comment_id,
            commentInfo.parent_comment_id,
            commentInfo.user_id,
            commentInfo.song_id,
            commentInfo.content,
            commentInfo.time,
            commentInfo.like_count,
            commentInfo.comment_type
        ]));
        let p2 = dataManager.user.insertCollection(userInfoList.map(userInfo => [
            userInfo.user_id,
            userInfo.user_type,
            userInfo.nickname,
            userInfo.avatar_url,
        ]));
        await Promise.all([p1, p2])
        // console.log("INSERT comment and user finished");

        // console.log(commentResult.body.more, comments.length, commentInfoList.length);

        // 判断是否还有下一页
        if (commentResult.body.more && comments.length > 0) {
            // 更新 progress
            progress.currentTime = comments[comments.length - 1].time;
            if (progress.maxTime == progress.minTime) { // minTime = maxTime 代表这是本轮爬取的第一次
                progress.maxTime = comments[0].time;
            }
            progress.currentStatus = 1; // 0-等待爬取/增量爬取 1-爬取中 2-完成
            // 更新 queryParams
            queryParams.before = progress.currentTime;
            progress.total = commentResult.body.total;
        } else {
            isFinish = true;
            console.log(`comment: ${songId} done\n`);
            progress.currentStatus = 2; // 0-等待爬取/增量爬取 1-爬取中 2-完成
            if (progress.maxTime == 0) { // 第一次爬取 且 没有分页的情况
                progress.maxTime = comments[0] ? (comments[0].time || 0) : 0;
            }
            progress.minTime = progress.maxTime; // minTime = maxTime 代表这一轮爬取完成了
            progress.currentTime = progress.maxTime; // 可有可无
        }

        // progress更新到数据库中
        let commentProgressInfo = {
            max_time: progress.maxTime,
            min_time: progress.minTime,
            current_time: progress.currentTime,
            current_status: progress.currentStatus,
            total: progress.total,
        };
        console.log("dataManager.comment_progress.update")
        let p3 = dataManager.comment_progress.update(commentProgressInfo, songId);
        await p3
        // console.log("UPDATE comment_progress");

        // console.log("sleepUtils.sleep")
        // await sleepUtils.sleep(global.sleepTime);
    }
}

function getCommitInfoForInsert(songId, comment, commentType) {
    return {
        comment_id: comment.commentId,
        parent_comment_id: comment.parentCommentId,
        user_id: comment.user ? comment.user.userId : null,
        song_id: songId,
        content: comment.content || "", // 有些 comment 的 content 为 null
        time: comment.time,
        like_count: comment.likedCount,
        comment_type: commentType, // 评论类型 0-comments 1-hotComments 2-topComments
    }
}

function getUserInfoForInsert(user) {
    var shortAvatarUrlUrl = user.avatarUrl.match(/^http:\/\/p\d+\.music\.126\.net\/(.*?)$/);
    shortAvatarUrlUrl = shortAvatarUrlUrl ? shortAvatarUrlUrl[1] : user.avatarUrl;
    return {
        user_id: user.userId,
        user_type: user.userType,
        nickname: user.nickname,
        avatar_url: shortAvatarUrlUrl || user.avatarUrl,
    }
}

module.exports = {
    fetchAll: fetchAll,
    fetch: fetch,
}