const fs = require('fs');
const path = require('path');

const sleepUtils = require('../../../utils/sleepUtils');
const dataManager = require('../dataManager');

const dbUtils = global.dbUtils;
const oldPlaylistUtils = require('./playlistUtils_old');

// refer:
// https://neteasecloudmusicapi-docs.4everland.app/
// https://github.com/Binaryify/NeteaseCloudMusicApi
const { top_playlist, top_playlist_highquality, related_playlist } = require('NeteaseCloudMusicApi');

/**
 * 这里有几种情况：
 *
 * top_playlist                         并保存相关歌单信息（只有id）
 * top_playlist_highquality             并保存相关歌单信息（只有id）
 * 还有就是从数据库中查出的待爬取歌单详情  并保存相关歌单信息（只有id）
 */

async function fetchTop({ args = {} }) {
    console.log("start fetching playlist ...");

    let limit = 50; // 默认为 50
    let page = 0;
    let hasMore = true;
    while (hasMore) {
        try {
            // https://neteasecloudmusicapi-docs.4everland.app/#/?id=%e6%ad%8c%e5%8d%95-%e7%bd%91%e5%8f%8b%e7%b2%be%e9%80%89%e7%a2%9f-
            var result = await top_playlist({ id: 2 });

            // https://neteasecloudmusicapi-docs.4everland.app/#/?id=%e8%8e%b7%e5%8f%96%e7%b2%be%e5%93%81%e6%ad%8c%e5%8d%95
            // var result = await top_playlist_highquality({});

            // 调用此接口,传入歌单 id 可获取相关歌单
            // https://neteasecloudmusicapi-docs.4everland.app/#/?id=%e7%9b%b8%e5%85%b3%e6%ad%8c%e5%8d%95%e6%8e%a8%e8%8d%90
            var result = await related_playlist({ id: 809945533 });

            // fs.writeFileSync(path.join(__dirname, "../../temp", `test.json`), JSON.stringify(result.body));
        } catch (errors) {
            console.error("error", errors);
            await sleepUtils.sleep(10 * 1000);
        }

        // 针对每一个歌单调用相关歌单接口，然后加入待爬取歌单

        hasMore = result.more;

        let playlists = result.body.playlists;
        // console.log(playlists);

        for (let i in playlists) {
            await global.checkIsExit();
            const playlist = playlists[i];
            const playlistId = playlist.id;
            console.log(`${Number(i) + 1}/${playlists.length} | playlist: ${playlistId} | limit: ${limit}, page: ${page}`);
            try {
                await oldPlaylistUtils.fetch({ playlistId: playlist.id })
            } catch (err) {
                console.error(err);
            }
            // await sleepUtils.sleep(global.sleepTime);
        }

         // 更新 is_fetched_related_playlist 字段
    }
}

module.exports = {
    fetchTop: fetchTop,
}