/**
 * TCP Server (node export module)
 * CommonJS
 */
//引入net模块
const net = require('net')

const LOG_DEBUG_INFO = false

// 启动一个 TCP Server
const serverConfig = item.serverConfig
// 绑定 TCP Server 的 callback
tcpServer.createServer(serverConfig.port, {
    onConnection: (socket, port) => {
        // some code here ...
    },
    onListening: (address, port, family) => {
        // some code here ...
    },
    onData: (data) => {
        // some code here ...
    },
    onClose: () => {
        // some code here ...
    },
    onError: (err) => {
        // some code here ...
    },
})

module.exports = {
    /**
     *
     * @param {*} port
     * @param {*} param1
     *              onConnection(socket, port)
     *              onListening(address, port, family)
     *              onData(data)
     *              onClose()
     *              onError(err)
     */
    createServer(port, { onConnection, onListening, onData, onClose, onError }) {
        LOG_DEBUG_INFO && console.log("启动 TCP 服务器")
        //实例化一个服务器对象
        const server = new net.Server()

        //监听connection事件
        server.on('connection', function (socket) {
            LOG_DEBUG_INFO && console.log('新客户端接入')

            //监听data事件
            socket.on("data", function (data) {
                //打印数据
                LOG_DEBUG_INFO && console.log("接收到数据：[数据开始]" + data.toString() + "[数据结束]")

                if (typeof (onData) === "function") {
                    onData(data)
                }
            });

            // 客户端 TCP 连接异常终止时，捕获该 error 并销毁该 socket，避免服务端异常退出
            socket.on('error', (err) => {
                console.log('Socket error: ' + err);
                // 这里可以根据错误类型进行不同的处理，例如关闭连接，重试，记录日志等
                socket.destroy();
            });

            if (typeof (onConnection) === "function") {
                onConnection(socket, port)
            }
        })

        //设置监听端口
        server.listen(port)

        //设置监听时的回调函数
        server.on('listening', function () {
            //获取地址信息
            let address = server.address()
            //获取地址详细信息
            LOG_DEBUG_INFO && console.log(`TCP 服务器监听地址: ${address.address}, 端口: ${address.port}, 类型: ${address.family}`)

            if (typeof (onListening) === "function") {
                onListening(address.address, address.port, address.family)
            }
        })

        //设置关闭时的回调函数
        server.on('close', function () {
            LOG_DEBUG_INFO && console.log('服务已关闭')

            if (typeof (onClose) === "function") {
                onClose()
            }
        })

        //设置出错时的回调函数
        server.on('error', function (err) {
            LOG_DEBUG_INFO && console.log('服务运行异常', err)

            if (typeof (onError) === "function") {
                onError(err)
            }
        })
    }
}