/**
 * WebSocket Client (node export module)
 * CommonJS
 */
// 引入
const WebSocket = require('websocket').client

const LOG_PREFIX = "[websocket-client]"

const PRINT_LOG = false

function reconnect(client, url) {
    // 打印一条重连消息
    console.log('Reconnecting to ' + url);
    // 调用client的connect方法
    client.connect(url);
}

function createConn(uniqueId, url, { onConnect, onMessage, onClose, onError }) {
    PRINT_LOG && console.log(LOG_PREFIX, "createConn", uniqueId, url)
    // 创建新的连接前先把旧的连接关掉
    // closeConn(uniqueId)

    let client = new WebSocket()

    // readyState返回当前实例对象的当前状态
    /*
      共有四种状态
      CONNECTING: 值为0，表示正在连接
      OPEN:  值为1，表示连接成功，可以通信了
      CLOSING 值为2， 表示连接正在关闭
      CLOSED 值为3，表示连接已经关闭了，或者打开连接失败
    */

    client.on('connect', function (connection) {
        // 连接成功后，打印一条消息
        console.log(LOG_PREFIX, 'Connected to ' + url);

        // callback
        if (typeof (onConnect) === 'function') {
            onConnect(connection)
        }

        // 监听消息事件
        connection.on('message', function (message) {
            // 收到消息后，打印出来
            let data = message.utf8Data
            PRINT_LOG && console.log(LOG_PREFIX, 'Received: ' + data.substring(0, 20) + ' ...');

            // callback
            if (typeof (onMessage) === 'function') {
                onMessage(data)
            }
        });

        // 监听关闭事件
        connection.on('close', function (reasonCode, description) {
            // 连接关闭后，打印一条消息
            console.log('Disconnected from ' + url);

            setTimeout(() => { // 重连
                reconnect(client, url)
            }, 1000)

            // callback
            if (typeof (onClose) === 'function') {
                onClose()
            }
        });

        // // 关闭连接
        // setTimeout(() => {
        //     connection.close()
        //     console.log("Close connection ...")
        // }, 2000)
    });


    // 监听错误事件
    client.on('connectFailed', function (error) {
        // 连接失败后，打印一条错误信息
        console.error('Connect Error: ' + error.toString());

        setTimeout(() => { // 重连
            reconnect(client, url)
        }, 1000)

        // callback
        if (typeof (onError) === 'function') {
            onError(error)
        }
    });

    client.connect(url)

    return client
}

// async function sendMessage(uniqueId, message) {
//     PRINT_LOG && console.log(LOG_PREFIX, "sendMessage", message)
//     let data = JSON.stringify(message)
//     if (wsInstance && wsInstance.send) {
//         while (wsInstance && wsInstance.readyState != 1) {
//             PRINT_LOG && console.warn("websocket 连接尚未准备好，等一会儿...", wsInstance.readyState)
//             await new Promise((resolve) => setTimeout(resolve, 500))
//         }
//         wsInstance.send(data)
//         return
//     }
//     console.warn("wsInstance.sendMessage is not defined", wsInstance, wsInstance.send)

// }

// function closeConn(uniqueId) {
//     wsInstance && wsInstance.close && wsInstance.close()
// }

module.exports = {
    createConn,
    // sendMessage,
    // closeConn,
}